#!/usr/bin/python3

# Licensed under AGPLv3+

import sys
import os

sys.path.append(os.path.dirname(__file__))
from lib.guicfg import *


def main() -> None:
    cfg = readcfg()
    guiconfigurator(cfg)

if __name__ == "__main__":
    main()

# vi: tabstop=4 shiftwidth=4 expandtab



from lib.locale import translation

translation["ru"] = {
    # Эмоции
    "Anger " : "Злость",
    "Contm." : "Презр.",
    "Disgu." : "Отврщ.",
    "Fear  " : "Страх ",
    "Happs." : "Радст.",
    "Neutr." : "Нейтр.",
    "Sadns." : "Грусть",
    "Surpr." : "Удивл.",

    # Всё остальное в алфавитном порядке
    "Analyze all faces" : "Анализировать все лица",
    "Beep on warning (depend on system settings)" : "Звуковое предупреждение (зависит от системных настроек)",
    "Can't open camera {}" : "Не удаётся открыть камеру {}",
    "Can't read camera image" : "Не удаётся получить изображение с камеры",
    "Camera number" : "Номер камеры",
    "Color Chooser" : "Выбор цвета",
    "Common settings" : "Общие настройки",
    "Config saved" : "Настройки сохранены",
    "Error" : "Ошибка",
    "FPS" : "Кадры в секунду",
    "Limits settings" : "Настройки пределов",
    "Lower limits" : "Нижние границы",
    "Reset" : "Сбросить",
    "Save" : "Сохранить",
    "Scanning directory {} for statistics" : "Открываю статистику в каталоге {}",
    "Show camera picture" : "Показывать изображение с камеры",
    "Show warning" : "Показывать предупреждение",
    "Trying to read the config file " : "Пробую прочитать файл настроек ",
    "Video height" : "Высота видео",
    "Video width" : "Ширина видео",
    "Warning color" : "Цвет предупреждения",
    "Warning delay in seconds" : "Задержка предупреждения в секундах",
    "Warning window size" : "Размер окна с предупреждением",
    "Warning window X pos" : "Координата X предупреждения",
    "Warning window Y pos" : "Координата Y предупреждения",
    "Write stat file" : "Сохранять статистику в файл",
    "Writing to the config file " : "Записываю файл настроек ",
    "Upper limits" : "Верхние границы",
}

# vi: tabstop=4 shiftwidth=4 expandtab



#!/usr/bin/python3

# Licensed under AGPLv3+

from tkinter import *
from tkinter.colorchooser import askcolor
import tkinter.ttk as ttk
import copy
import time

from lib.i18n import _
from lib.config import *


# Internally used class
class counter(object):
    def __init__(self, v=0):
        if type(v) != int:
            v = 0
        self.v = v

    def get(self):
        return self.v

    def inc(self):
        self.v += 1
        return self.v - 1


# GUI configurator class
class guiconfigurator:
    def __init__(self, cfg):
        self.cfg = cfg
        self.__create_window()

    def __close_window(self):
        self.running = False

    # Main function of the class
    def __create_window(self):

        self.savedentcolor = None  # Default entry background color
        self.running = True

        root = Tk()
        root.protocol("WM_DELETE_WINDOW", self.__close_window)
        nb = ttk.Notebook(root)
        nb.pack(side=TOP)

        # Create used tabs
        misctab = ttk.Frame(nb)
        limtab = ttk.Frame(nb)

        nb.add(misctab, text=_("Common settings"))
        nb.add(limtab, text=_("Limits settings"))

        #
        # Common settings
        #
        row = counter()
        Label(misctab, text=_("Camera number")).grid(
                row=row.get(), column=0, sticky=W)
        self.camera_dev = Entry(misctab, width=4)
        self.camera_dev.grid(row=row.inc(), column=1, sticky=W)

        # Once remember default Entry background color
        self.savedentcolor = self.camera_dev.cget("highlightbackground")

        self.res = [0, 0]
        Label(misctab, text=_("Video width")).grid(
                row=row.get(), column=0, sticky=W)
        self.res[0] = Entry(misctab, width=4)
        self.res[0].grid(row=row.inc(), column=1, sticky=W)

        Label(misctab, text=_("Video height")).grid(
                row=row.get(), column=0, sticky=W)
        self.res[1] = Entry(misctab, width=4)
        self.res[1].grid(row=row.inc(), column=1, sticky=W)

        Label(misctab, text=_("FPS")).grid(row=row.get(), column=0, sticky=W)
        self.fps = Entry(misctab, width=4)
        self.fps.grid(row=row.inc(), column=1, sticky=W)

        Label(misctab, text=_("Warning delay in seconds")).grid(row=row.get(), column=0, sticky=W)
        self.wdelay = Entry(misctab, width=4)
        self.wdelay.grid(row=row.inc(), column=1, sticky=W)

        self.showcap = BooleanVar()
        Checkbutton(
                misctab, text=_("Show camera picture"), variable=self.showcap,
                onvalue=1, offvalue=0).grid(row=row.inc(), column=0, sticky=W)

        self.allfaces = BooleanVar()
        Checkbutton(
                misctab, text=_("Analyze all faces"), variable=self.allfaces,
                onvalue=1, offvalue=0).grid(row=row.inc(), column=0, sticky=W)

        self.writestat = BooleanVar()
        Checkbutton(
                misctab, text=_("Write stat file"), variable=self.writestat,
                onvalue=1, offvalue=0).grid(row=row.inc(), column=0, sticky=W)

        self.beepwarn = BooleanVar()
        Checkbutton(
                misctab, text=_("Beep on warning (depend on system settings)"), variable=self.beepwarn,
                command=self.endis,
                onvalue=1, offvalue=0).grid(row=row.inc(), column=0, sticky=W)

        self.showwarn = BooleanVar()
        Checkbutton(
                misctab, text=_("Show warning"), variable=self.showwarn,
                command=self.endis,
                onvalue=1, offvalue=0).grid(row=row.inc(), column=0, sticky=W)

        Label(misctab, text=_("Warning window size")).grid(
                row=row.get(), column=0, sticky=W)
        self.wsize = Entry(misctab, width=4)
        self.wsize.grid(row=row.inc(), column=1, sticky=W)

        self.wpos = [0, 0]
        Label(misctab, text=_("Warning window X pos")).grid(
                row=row.get(), column=0, sticky=W)
        self.wpos[0] = Entry(misctab, width=4)
        self.wpos[0].grid(row=row.inc(), column=1, sticky=W)

        Label(misctab, text=_("Warning window Y pos")).grid(
                row=row.get(), column=0, sticky=W)
        self.wpos[1] = Entry(misctab, width=4)
        self.wpos[1].grid(row=row.inc(), column=1, sticky=W)

        Button(misctab, text=_("Warning color"), command=self.wcolor).grid(
                row=row.inc(), column=0)

        #
        # Limits settings
        #
        row = counter()

        Label(limtab, text=_("Upper limits")).grid(
                row=row.inc(), column=0, columnspan=8, sticky=W+E)

        # Checkbuttons for max limit
        self.wmax = []
        self.wmaxen = []
        column = counter()
        for i, ign in enumerate(self.cfg.wmax):
            self.wmaxen.append(None)
            self.wmaxen[i] = BooleanVar()
            Checkbutton(
                    limtab, text=emotions[i], variable=self.wmaxen[i],
                    onvalue=1, offvalue=0, command=self.endis).grid(
                    row=row.get(), column=column.inc())
        # Scales for max limit
        row.inc()
        column = counter()
        for i, ign in enumerate(self.cfg.wmax):
            self.wmax.append(Scale(
                    limtab, orient=VERTICAL, length=300,
                    from_=5, to=-5, tickinterval=0.0, resolution=0.1))
            self.wmax[i].grid(row=row.get(), column=column.inc())

        row.inc()

        Label(limtab, text=_("Lower limits")).grid(
                row=row.inc(), column=0, columnspan=8, sticky=W+E)
        # Checkbuttons for min limit
        self.wmin = []
        self.wminen = []
        column = counter()
        for i, ign in enumerate(self.cfg.wmin):
            self.wminen.append(None)
            self.wminen[i] = BooleanVar()
            Checkbutton(
                    limtab, text=emotions[i], variable=self.wminen[i],
                    onvalue=1, offvalue=0, command=self.endis).grid(
                    row=row.get(), column=column.inc())
        # Scales for min limit
        row.inc()
        column = counter()
        for i, ign in enumerate(self.cfg.wmin):
            self.wmin.append(Scale(
                    limtab, orient=VERTICAL, length=300, from_=5, to=-5,
                    tickinterval=0.0, resolution=0.1))
            self.wmin[i].grid(row=row.get(), column=column.inc())

        row.inc()

        self.note = Label(root, text="")
        self.note.pack(side=TOP, anchor=W)
        Button(root, text=_("Save"), command=self.save_guicfg).pack(side=LEFT)
        Button(root, text=_("Reset"), command=self.cfg2gui).pack(side=LEFT)

        self.cfg2gui()  # Fill with actual values

        self.endis()  # Update enable/disable status of interface elementes

        while self.running is True:
            root.update_idletasks()
            root.update()
            time.sleep(0.05)

    # Scroll enabler/disabler
    def endis(self,):
        if (self.showwarn.get() is True):
            self.wsize.configure(state=NORMAL)
            self.wpos[0].configure(state=NORMAL)
            self.wpos[1].configure(state=NORMAL)
        else:
            self.wsize.configure(state=DISABLED)
            self.wpos[0].configure(state=DISABLED)
            self.wpos[1].configure(state=DISABLED)

        for i, ign in enumerate(self.cfg.wmax):
            if (self.wmaxen[i].get() is True):
                self.wmax[i].config(state=NORMAL, relief=RAISED)
            else:
                self.wmax[i].config(state=DISABLED, relief=FLAT)

        for i, ign in enumerate(self.cfg.wmin):
            if (self.wminen[i].get() is True):
                self.wmin[i].config(state=NORMAL, relief=RAISED)
            else:
                self.wmin[i].config(state=DISABLED, relief=FLAT)

    # Color chooser
    def wcolor(self,):
        colors = askcolor(initialcolor=self.wcolor, title=_("Color Chooser"))
        self.wcolor = colors[0]

    # Action on save button
    def save_guicfg(self) -> None:
        errorflag, newcfg = self.get_gui_cfg()

        # Show warning if needed
        if errorflag == 0:
            writecfg(newcfg)
            self.cfg = newcfg
            self.note.config(text=_("Config saved"), fg='green')
        else:
            self.note.config(text=_("Error"), fg='red')

    # Update cfg structure with configuration from GUI
    def get_gui_cfg(self):
        newcfg = copy.copy(self.cfg)
        errorflag = 0

        # Common settings
        try:
            newcfg.camera_dev = int(self.camera_dev.get())
            self.camera_dev.configure(
                    highlightbackground=self.savedentcolor,
                    highlightcolor=self.savedentcolor)
        except Exception as e:
            self.camera_dev.configure(
                    highlightbackground='red', highlightcolor='red')
            errorflag = 1

        try:
            newcfg.res[0] = int(self.res[0].get())
            self.res[0].configure(
                    highlightbackground=self.savedentcolor,
                    highlightcolor=self.savedentcolor)
        except Exception as e:
            self.res[0].configure(
                    highlightbackground='red', highlightcolor='red')
            errorflag = 1

        try:
            newcfg.res[1] = int(self.res[1].get())
            self.res[1].configure(
                    highlightbackground=self.savedentcolor,
                    highlightcolor=self.savedentcolor)
        except Exception as e:
            self.res[1].configure(
                    highlightbackground='red', highlightcolor='red')
            errorflag = 1

        try:
            newcfg.fps = float(self.fps.get())
            self.fps.configure(
                    highlightbackground=self.savedentcolor,
                    highlightcolor=self.savedentcolor)
        except Exception as e:
            self.fps.configure(highlightbackground='red', highlightcolor='red')
            errorflag = 1

        try:
            newcfg.wdelay = int(self.wdelay.get())
            self.wdelay.configure(
                    highlightbackground=self.savedentcolor,
                    highlightcolor=self.savedentcolor)
        except Exception as e:
            self.wdelay.configure(highlightbackground='red', highlightcolor='red')
            errorflag = 1

        newcfg.showcap = self.showcap.get()
        newcfg.allfaces = self.allfaces.get()
        newcfg.writestat = self.writestat.get()
        newcfg.showwarn = self.showwarn.get()
        newcfg.beepwarn = self.beepwarn.get()

        try:
            newcfg.wsize = int(self.wsize.get())
            self.wsize.configure(
                    highlightbackground=self.savedentcolor,
                    highlightcolor=self.savedentcolor)
        except Exception as e:
            self.wsize.configure(
                    highlightbackground='red', highlightcolor='red')
            errorflag = 1

        try:
            newcfg.wpos[0] = int(self.wpos[0].get())
            self.wpos[0].configure(
                    highlightbackground=self.savedentcolor,
                    highlightcolor=self.savedentcolor)
        except Exception as e:
            self.wpos[0].configure(
                    highlightbackground='red', highlightcolor='red')
            errorflag = 1

        try:
            newcfg.wpos[1] = int(self.wpos[1].get())
            self.wpos[1].configure(
                    highlightbackground=self.savedentcolor,
                    highlightcolor=self.savedentcolor)
        except Exception as e:
            self.wpos[1].configure(highlightbackground='red', highlightcolor='red')
            errorflag = 1

        newcfg.wcolor = self.wcolor

        # Emotion limits
        for i, ign in enumerate(self.cfg.wmax):
            newcfg.wmaxen[i] = self.wmaxen[i].get()
            newcfg.wmax[i] = self.wmax[i].get()

        for i, ign in enumerate(self.cfg.wmin):
            newcfg.wminen[i] = self.wminen[i].get()
            newcfg.wmin[i] = self.wmin[i].get()

        return errorflag, newcfg

    # Fill gui fields with the read config
    def cfg2gui(self):
        # Common settings
        self.camera_dev.delete(0, END)
        self.camera_dev.insert(0, str(self.cfg.camera_dev))
        self.res[0].delete(0, END)
        self.res[0].insert(0, str(self.cfg.res[0]))
        self.res[1].delete(0, END)
        self.res[1].insert(0, str(self.cfg.res[1]))
        self.fps.delete(0, END)
        self.fps.insert(0, str(self.cfg.fps))
        self.wdelay.delete(0, END)
        self.wdelay.insert(0, str(self.cfg.wdelay))
        self.showcap.set(1 if self.cfg.showcap else 0)
        self.allfaces.set(1 if self.cfg.allfaces else 0)
        self.writestat.set(1 if self.cfg.writestat else 0)
        self.showwarn.set(1 if self.cfg.showwarn else 0)
        self.beepwarn.set(1 if self.cfg.beepwarn else 0)
        self.wsize.delete(0, END)
        self.wsize.insert(0, str(self.cfg.wsize))
        self.wpos[0].delete(0, END)
        self.wpos[0].insert(0, str(self.cfg.wpos[0]))
        self.wpos[1].delete(0, END)
        self.wpos[1].insert(0, str(self.cfg.wpos[1]))
        self.wcolor = self.cfg.wcolor

        # Emotion limits
        for i, val in enumerate(self.cfg.wmaxen):
            self.wmaxen[i].set(val)
            self.wmax[i].set(self.cfg.wmax[i])

        for i, val in enumerate(self.cfg.wminen):
            self.wminen[i].set(val)
            self.wmin[i].set(self.cfg.wmin[i])

        # Reset note, if it was
        self.note.config(text="")

# vi: tabstop=4 shiftwidth=4 expandtab



import locale
import sys
import os
from lib.locale import translation
from lib.locale.ru import *

if sys.platform != 'darwin':
    # The locale.getdefaultlocale() is obsolete, so this code should be updated
    curr_lang = locale.getdefaultlocale()[0].split('_')[0]
else:
    # Standard locale app doesn't work correct in the Mac OS, so use a hack
    sys_lang = os.popen("defaults read -g AppleLanguages").read()
    str_lang = sys_lang.translate({ord(i): None for i in '()"\n '})
    curr_lang = str_lang.split('-')[0]


def _(arg):
    if curr_lang in translation:
        if arg in translation[curr_lang]:
            return translation[curr_lang][arg]
    return arg



#!/usr/bin/python3

# Licensed under AGPLv3+

import os
import sys

os.environ["OPENCV_LOG_LEVEL"]="FATAL" # Suppress OpenCV messages

import cv2
import numpy
from datetime import datetime,timedelta
import platformdirs
from hsemotion_onnx.facial_emotions import HSEmotionRecognizer

sys.path.append(os.path.dirname(__file__))
from lib.centerface import CenterFace
from lib.i18n import _
from lib.config import *
from lib.cam import *


def writestat(cfg, i, scores) -> None:
    now = datetime.now()
    print(f'{now.strftime("%H:%M:%S")} ', end='')

    # Print scores
    emax = numpy.argmax(scores)
    print(f'[{i}]: {emotions[emax]}; ', end='')
    for e in range(len(emotions)):
        print(f'{emotions[e]}: {scores[e]:4.1f}', end='')
        if e < (len(emotions) - 1):
            print(f', ', end='')
        else:
            print('')

    # Write to logs
    if cfg.writestat:
        if not os.path.exists(platformdirs.user_log_dir(ANAME)):
            os.makedirs(platformdirs.user_log_dir(ANAME))
        fp = open(platformdirs.user_log_dir(ANAME) + "/" + now.strftime("%Y.%m.%d"), 'a')
        str = now.strftime("%H:%M:%S")
        for e in range(len(emotions)):
            str = str + " %4.1lf" % (scores[e])
        str = str + "\n"
        fp.write(str)
        fp.close()

#  Parameters:  configuration, scores, warning was set before
#               waring action was active before, last waring was turned on time
def warn_actions(cfg, scores, wws, wwact, wstime):
    # Check warning state and notify user
    wname = "Face warn"
    ws = False # Warning state flag
    for e in range(len(emotions)):
        if cfg.wminen[e] and scores[e] < cfg.wmin[e]:
            ws = True
            break
        if cfg.wmaxen[e] and scores[e] > cfg.wmax[e]:
            ws = True
            break

    if wws is False and ws is True: # If warning state is switched on
        wstime = datetime.now() # Remember time

    # Set/reset warning action depending on warning state and it's timeout
    if ws and wstime + timedelta(seconds=cfg.wdelay) <= datetime.now():
        wact = True
    else:
        wact = False

    try:
        # Show warning window in case of:
        #     it's warning condition, it's on in cfg, has no warning window yet
        if wact and cfg.showwarn and not wwact:
            # Use OpenCV to avoid excess dependencies

            w = cfg.wsize
            h = cfg.wsize
            if sys.platform == 'darwin' and w < 200:
                # Mac OS doesn't allow windows width less than 200, so...
                w = 200

            wimg = numpy.zeros((h, w, 3), numpy.uint8)
            wimg = cv2.rectangle(wimg, (0, 0), (w - 1, h - 1), cfg.wcolor[::-1], -1)
            font = cv2.FONT_HERSHEY_SIMPLEX
            text = "!"
            linew = int(h / 32)
            textsize = cv2.getTextSize(text, font, 1, linew)[0]
            textx = int((wimg.shape[1] - textsize[0]) / 2)
            texty = int((wimg.shape[0] + textsize[1]) / 2)
            cv2.putText(wimg, "!", (textx, texty), font, 1, (0, 0, 0), linew)
            cv2.namedWindow(wname, cv2.WINDOW_NORMAL | cv2.WINDOW_FREERATIO | cv2.WINDOW_GUI_NORMAL)
            cv2.resizeWindow(wname, w, h)
            cv2.moveWindow(wname, cfg.wpos[0], cfg.wpos[1])
            cv2.setWindowProperty(wname, cv2.WND_PROP_TOPMOST, 1)
            cv2.imshow(wname, wimg)
        # Destroy warning window in case of:
        #     it's not to be shown (cfg and condition) and has previous window
        elif not (cfg.showwarn and wact) and wwact:
            cv2.destroyWindow(wname)

        if wact and cfg.beepwarn:
            # Generate system beep
            print("\a", end="")
    except Exception as exc:
        print(f'Warning: {exc}')

    return ws, wact, wstime


def main() -> None:
    MODEL_NAME='enet_b0_8_best_vgaf'
    cfg = readcfg()
    cam = cam_class(cfg)
    bestface = 0 # If configured, here is index of the most intresting face
    wws = False # Warning condition was set
    wwact = False # Warning windows was shown flag
    wstime = datetime.now() # Warning condition set time

    ret, cap = cam.find_camera()
    if ret is False:
        return -1

    # Set main window properties
    cv2.namedWindow('Video', cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)

    # Set neural networks
    centerface = CenterFace()
    fer = HSEmotionRecognizer(MODEL_NAME)

    while True:
        ret, image_bgr = cam.get_next_frame()
        if not ret:
            print(_("Can't read camera image"))
            return -1

        image = cv2.cvtColor(image_bgr, cv2.COLOR_BGR2RGB)
        bounding_boxes, ign = centerface(image_bgr, image_bgr.shape[0], image_bgr.shape[1], threshold=0.35)

        if not cfg.allfaces:
            # Find biggest (i.e. closer to camera) face
            areas = (bounding_boxes[:, 2] - bounding_boxes[:, 0] + 1) * (bounding_boxes[:, 3] - bounding_boxes[:, 1] + 1)
            if areas.size != 0:
                bestface = areas.argmax()

        for i in range(len(bounding_boxes)):
            bbox = bounding_boxes[i]
            x1, y1, x2, y2 = [round(b) for b in bbox[0:4]]
            if (x1 <= 0): x1 = 0
            if (y1 <= 0): y1 = 0
            if (x2 >= image_bgr.shape[1]): x2 = image_bgr.shape[1] - 1
            if (y2 >= image_bgr.shape[0]): y2 = image_bgr.shape[0] - 1

            # Just draw border around not selected face
            if not cfg.allfaces and i != bestface:
                cv2.rectangle(image_bgr, (x1, y1), (x2, y2), (255, 255, 255), 1)
                continue

            face_img = image[y1:y2, x1:x2]
            emotion, scores = fer.predict_emotions(face_img,logits=True)
            cv2.rectangle(image_bgr, (x1, y1), (x2, y2), (255, 0, 0), 1)

            wws, wwact, wstime = warn_actions(cfg, scores, wws, wwact, wstime)
            writestat(cfg, i, scores)

        if cfg.showcap:
            # Exit if main window was closed by user
            if cv2.getWindowProperty('Video', cv2.WND_PROP_VISIBLE) < 1:
                break

            # Show captured image with the detected box
            cv2.imshow('Video', image_bgr)

            if cv2.waitKey(1) & 0xFF == ord('q'):
                break


if __name__ == "__main__":
    main()

# vi: tabstop=4 shiftwidth=4 expandtab



# Licensed under AGPLv3+

import configparser
import platformdirs
import os

from lib.i18n import _


ANAME = "sevimon"
emotions = (_("Anger "), _("Contm."), _("Disgu."), _("Fear  "), _("Happs."), _("Neutr."), _("Sadns."), _("Surpr."))


class configclass:
    def __init__(self,):
        pass

# Simple sanity check for input string
# Args: input string, default array
def get_check_ints(sec, name, defv):
    try:
        s = sec.get(name)

        if not set(s).issubset(set(" \t,0123456789")):
            return defv

        res = [int(ele) for ele in s.split(',')]

        if (len(res) != len(defv)):
            return defv
    except Exception as exc:
        return defv

    return res


def get_check_floats(sec, name, defv):
    try:
        s = sec.get(name)

        if not set(s).issubset(set(" \t.,0123456789")):
            return defv

        res = [float(ele) for ele in s.split(',')]

        if (len(res) != len(defv)):
            return defv
    except Exception as exc:
        return defv

    return res


def get_check_bools(sec, name, defv):
    try:
        s = sec.get(name)

        if not set(s).issubset(set(" \t,TrueFalse")):
            return defv

        s = s.replace(' ', '').replace('\t', '')
        tmps = [ele for ele in s.split(',')]

        res = []
        for i in tmps:
            if i == "True":
                res.append(True)
            elif i == "False":
                res.append(False)
            else:
                return defv

        if (len(res) != len(defv)):
            return defv
    except Exception as exc:
        return defv

    return res


def readcfg() -> configclass:
    cfg = configclass()

    config = configparser.ConfigParser()
    configname = platformdirs.user_config_dir(ANAME) + "/" + ANAME + ".cfg"
    print(_("Trying to read the config file ") + configname)
    try:
        config.read(configname)
    except Exception as exc:
        pass

    try:
        section = config["common"]
    except:
        config.add_section("common")
        section = config["common"]
        pass

    cfg.camera_dev = get_check_ints(section,"camera_dev", [0])[0]
    cfg.res = get_check_ints(section,"res", [640, 480])
    cfg.fps = get_check_floats(section,"fps", [1.0])[0]
    cfg.wdelay = get_check_ints(section,"wdelay", [0])[0]

    cfg.showcap = get_check_bools(section,"showcap", [True])[0]
    cfg.allfaces = get_check_bools(section,"allfaces", [False])[0]

    cfg.writestat = get_check_bools(section,"writestat", [True])[0]

    cfg.wsize = get_check_ints(section,"wsize", [95])[0]
    cfg.wpos = get_check_ints(section,"wpos", [300, 150])
    cfg.wcolor = tuple(get_check_ints(section,"wcolor", [0, 255, 0]))

    cfg.showwarn = get_check_bools(section,"showwarn", [True])[0]
    cfg.beepwarn = get_check_bools(section,"beepwarn", [False])[0]

    cfg.wmax = get_check_floats(section,"wmax",
            [4.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.5])
    cfg.wmin = get_check_floats(section,"wmin",
            [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])

    cfg.wmaxen = get_check_bools(section,"wmaxen",
            [True, False, False, False, False, False, False, True])
    cfg.wminen = get_check_bools(section,"wminen",
            [False, False, False, False, False, False, False, False])

    return cfg


def writecfg(cfg) -> None:
    config = configparser.ConfigParser()
    configname = platformdirs.user_config_dir(ANAME) + "/" + ANAME + ".cfg"
    os.makedirs(platformdirs.user_config_dir(ANAME), exist_ok=True)

    with open(configname, 'w') as configfile:
        config.add_section("common")
        section = config["common"]
        section["camera_dev"] = str(cfg.camera_dev)
        section["res"] = str(cfg.res).replace('[', '').replace(']', '')
        section["fps"] = str(cfg.fps)
        section["wdelay"] = str(cfg.wdelay)
        section["showcap"] = str(cfg.showcap)
        section["allfaces"] = str(cfg.allfaces)
        section["writestat"] = str(cfg.writestat)
        section["wsize"] = str(cfg.wsize)
        section["wpos"] = str(cfg.wpos).replace('[', '').replace(']', '')
        section["wcolor"] = str(cfg.wcolor).replace('(', '').replace(')', '')
        section["showwarn"] = str(cfg.showwarn)
        section["beepwarn"] = str(cfg.beepwarn)
        section["wmax"] = str(cfg.wmax).replace('[', '').replace(']', '')
        section["wmin"] = str(cfg.wmin).replace('[', '').replace(']', '')
        section["wmaxen"] = str(cfg.wmaxen).replace('[', '').replace(']', '')
        section["wminen"] = str(cfg.wminen).replace('[', '').replace(']', '')

        if not os.path.exists(platformdirs.user_config_dir(ANAME)):
            os.makedirs(platformdirs.user_config_dir(ANAME))
        print(_("Writing to the config file ") + configname)
        config.write(configfile)

# vi: tabstop=4 shiftwidth=4 expandtab



import numpy as np
import cv2
import datetime

import os
from urllib import request
import platformdirs

# This code is a slightly modified https://github.com/Star-Clouds/CenterFace/blob/master/prj-python/centerface.py licensed under MIT.

class CenterFace(object):
    def __init__(self, landmarks=True):
        self.landmarks = landmarks
        appname = "sevimon"

        models_path=platformdirs.user_cache_dir(appname)
        if not os.path.exists(models_path):
            os.makedirs(models_path)
        if self.landmarks:
            filename = 'centerface.onnx'
        else:
            filename = 'cface.1k.onnx'
        filename=os.path.join(models_path,filename)
        if not os.path.isfile(filename):
             request.urlretrieve("https://github.com/Star-Clouds/CenterFace/raw/master/models/onnx/centerface.onnx", filename)

        self.net = cv2.dnn.readNetFromONNX(filename)
        self.img_h_new, self.img_w_new, self.scale_h, self.scale_w = 0, 0, 0, 0

    def __call__(self, img, height, width, threshold=0.5):
        self.img_h_new, self.img_w_new, self.scale_h, self.scale_w = self.transform(height, width)
        return self.inference_opencv(img, threshold)

    def inference_opencv(self, img, threshold):
        blob = cv2.dnn.blobFromImage(img, scalefactor=1.0, size=(self.img_w_new, self.img_h_new), mean=(0, 0, 0), swapRB=True, crop=False)
        self.net.setInput(blob)
        begin = datetime.datetime.now()
        if self.landmarks:
            heatmap, scale, offset, lms = self.net.forward(["537", "538", "539", '540'])
        else:
            heatmap, scale, offset = self.net.forward(["535", "536", "537"])
        end = datetime.datetime.now()
#        print("cpu times = ", end - begin)
        return self.postprocess(heatmap, lms, offset, scale, threshold)

    def transform(self, h, w):
        img_h_new, img_w_new = int(np.ceil(h / 32) * 32), int(np.ceil(w / 32) * 32)
        scale_h, scale_w = img_h_new / h, img_w_new / w
        return img_h_new, img_w_new, scale_h, scale_w

    def postprocess(self, heatmap, lms, offset, scale, threshold):
        if self.landmarks:
            dets, lms = self.decode(heatmap, scale, offset, lms, (self.img_h_new, self.img_w_new), threshold=threshold)
        else:
            dets = self.decode(heatmap, scale, offset, None, (self.img_h_new, self.img_w_new), threshold=threshold)
        if len(dets) > 0:
            dets[:, 0:4:2], dets[:, 1:4:2] = dets[:, 0:4:2] / self.scale_w, dets[:, 1:4:2] / self.scale_h
            if self.landmarks:
                lms[:, 0:10:2], lms[:, 1:10:2] = lms[:, 0:10:2] / self.scale_w, lms[:, 1:10:2] / self.scale_h
        else:
            dets = np.empty(shape=[0, 5], dtype=np.float32)
            if self.landmarks:
                lms = np.empty(shape=[0, 10], dtype=np.float32)
        if self.landmarks:
            return dets, lms
        else:
            return dets

    def decode(self, heatmap, scale, offset, landmark, size, threshold=0.1):
        heatmap = np.squeeze(heatmap)
        scale0, scale1 = scale[0, 0, :, :], scale[0, 1, :, :]
        offset0, offset1 = offset[0, 0, :, :], offset[0, 1, :, :]
        c0, c1 = np.where(heatmap > threshold)
        if self.landmarks:
            boxes, lms = [], []
        else:
            boxes = []
        if len(c0) > 0:
            for i in range(len(c0)):
                s0, s1 = np.exp(scale0[c0[i], c1[i]]) * 4, np.exp(scale1[c0[i], c1[i]]) * 4
                o0, o1 = offset0[c0[i], c1[i]], offset1[c0[i], c1[i]]
                s = heatmap[c0[i], c1[i]]
                x1, y1 = max(0, (c1[i] + o1 + 0.5) * 4 - s1 / 2), max(0, (c0[i] + o0 + 0.5) * 4 - s0 / 2)
                x1, y1 = min(x1, size[1]), min(y1, size[0])
                boxes.append([x1, y1, min(x1 + s1, size[1]), min(y1 + s0, size[0]), s])
                if self.landmarks:
                    lm = []
                    for j in range(5):
                        lm.append(landmark[0, j * 2 + 1, c0[i], c1[i]] * s1 + x1)
                        lm.append(landmark[0, j * 2, c0[i], c1[i]] * s0 + y1)
                    lms.append(lm)
            boxes = np.asarray(boxes, dtype=np.float32)
            keep = self.nms(boxes[:, :4], boxes[:, 4], 0.3)
            boxes = boxes[keep, :]
            if self.landmarks:
                lms = np.asarray(lms, dtype=np.float32)
                lms = lms[keep, :]
        if self.landmarks:
            return boxes, lms
        else:
            return boxes

    def nms(self, boxes, scores, nms_thresh):
        x1 = boxes[:, 0]
        y1 = boxes[:, 1]
        x2 = boxes[:, 2]
        y2 = boxes[:, 3]
        areas = (x2 - x1 + 1) * (y2 - y1 + 1)
        order = np.argsort(scores)[::-1]
        num_detections = boxes.shape[0]
        suppressed = np.zeros((num_detections,), dtype=bool)

        keep = []
        for _i in range(num_detections):
            i = order[_i]
            if suppressed[i]:
                continue
            keep.append(i)

            ix1 = x1[i]
            iy1 = y1[i]
            ix2 = x2[i]
            iy2 = y2[i]
            iarea = areas[i]

            for _j in range(_i + 1, num_detections):
                j = order[_j]
                if suppressed[j]:
                    continue

                xx1 = max(ix1, x1[j])
                yy1 = max(iy1, y1[j])
                xx2 = min(ix2, x2[j])
                yy2 = min(iy2, y2[j])
                w = max(0, xx2 - xx1 + 1)
                h = max(0, yy2 - yy1 + 1)

                inter = w * h
                ovr = inter / (iarea + areas[j] - inter)
                if ovr >= nms_thresh:
                    suppressed[j] = True

        return keep



import cv2

from lib.i18n import _


class cam_class():
    skip_frames = 0 # How many frames to skip to achive configured FPS
    cfg = None # Config structure
    cap = None

    def __init__(self, cfg):
        self.cfg = cfg

	# Find and open camera
    def find_camera(self):
        self.cap = cv2.VideoCapture(self.cfg.camera_dev)
        ret = self.cap is not None and self.cap.isOpened()
        if ret is False:
            print (_("Can't open camera {}").format(self.cfg.camera_dev))

        self.cap.set(cv2.CAP_PROP_FRAME_WIDTH, self.cfg.res[0])
        self.cap.set(cv2.CAP_PROP_FRAME_HEIGHT, self.cfg.res[1])
        self.cap.set(cv2.CAP_PROP_FPS, self.cfg.fps * 2)  # FPS, multiplication by 2 because of hack to clean buffer
        real_fps = int(self.cap.get(5))  # Get actual FPS from hardware
        self.skip_frames = real_fps / self.cfg.fps - 1  # Is there a better way?

        return ret, self.cap

    # Get the next frame with required FPS
    def get_next_frame(self):
        for i in range(int(self.skip_frames)):
            self.cap.grab()
        return self.cap.read()

# vi: tabstop=4 shiftwidth=4 expandtab



#!/usr/bin/python3

# Licensed under AGPLv3

from matplotlib import pyplot as plt
import os
import sys
import configparser
import platformdirs

sys.path.append(os.path.dirname(__file__))
from lib.i18n import _
from lib.config import *


def main() -> None:

    x = []
    dtime = []
    ang = []
    cont = []
    disq = []
    fear = []
    happ = []
    neutr = []
    sad = []
    surp = []

    #cutlen = 16 # Hack to ignore seconds
    cutlen = 13 # Hack to ignore minutes

    # Read all files in the log dir
    dtold = None
    i = 0
    curx = 0
    logdir = platformdirs.user_log_dir(ANAME)
    print(_("Scanning directory {} for statistics").format(logdir))
    for logname in sorted(os.listdir(logdir)):
        date = logname
        year, mon, day = logname.split(".")

        with open(logdir + "/" + logname, 'r') as f:
            for record in f:
                hms, _ang, _cont, _disq, _fear, _happ, _neutr, _sad, _surp = record.split()
                hour, mn, sec = hms.split(":")
                dt = date + " " + hms
                dt = dt[:cutlen]
                if dt == dtold:
                    m_ang += float(_ang)
                    m_cont += float(_cont)
                    m_disq += float(_disq)
                    m_fear += float(_fear)
                    m_happ += float(_happ)
                    m_neutr += float(_neutr)
                    m_sad += float(_sad)
                    m_surp += float(_surp)
                    i = i + 1
                    continue
                else:
                    if not dtold == None:
                        dtime.append('{:02}-{:02} {:02}:{:02}'.format(o_mon, o_day, o_hour, o_mn))
                        ang.append(m_ang / i)
                        cont.append(m_cont / i)
                        disq.append(m_disq / i)
                        fear.append(m_fear / i)
                        happ.append(m_happ / i)
                        neutr.append(m_neutr / i)
                        sad.append(m_sad / i)
                        surp.append(m_surp / i)
                        x.append(curx)
                        curx += 1
                    o_year = int(year)
                    o_mon = int(mon)
                    o_day = int(day)
                    o_hour = int(hour)
                    o_mn = int(mn)
                    o_sec = int(sec)

                    m_ang = 0.0
                    m_cont = 0.0
                    m_disq = 0.0
                    m_fear = 0.0
                    m_happ = 0.0
                    m_neutr = 0.0
                    m_sad = 0.0
                    m_surp = 0.0

                    dtold = dt
                    i = 0

    # Finilize last interval
    if not i == 0:
        dtime.append('{:02}-{:02} {:02}:{:02}'.format(o_mon, o_day, o_hour, o_mn))
        ang.append(m_ang / i)
        cont.append(m_cont / i)
        disq.append(m_disq / i)
        fear.append(m_fear / i)
        happ.append(m_happ / i)
        neutr.append(m_neutr / i)
        sad.append(m_sad / i)
        surp.append(m_surp / i)
        x.append(curx)

    plt.plot(ang, color="red", label=emotions[0], markersize=1, linestyle='-')
    plt.plot(cont, color="green", label=emotions[1], markersize=1, linestyle='-')
    plt.plot(disq, color="blueviolet", label=emotions[2], markersize=1, linestyle='-')
    plt.plot(fear, color="brown", label=emotions[3], markersize=1, linestyle='-')
    plt.plot(happ, color="orange", label=emotions[4], markersize=1, linestyle='-')
    plt.plot(neutr, color="gray", label=emotions[5], markersize=1, linestyle='-')
    plt.plot(sad, color="black", label=emotions[6], markersize=1, linestyle='-')
    plt.plot(surp, color="royalblue", label=emotions[7], markersize=1, linestyle='-')

    plt.xticks(x, dtime,  rotation=80)

    plt.margins(x=0.0, y=0.0)
    plt.legend()
    plt.subplots_adjust(bottom=0.2, top=0.98, left=0.03, right=0.97)
    plt.show()


if __name__ == "__main__":
    main()

# vi: tabstop=4 shiftwidth=4 expandtab



from setuptools import setup, find_packages
import os

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name='sevimon',
    version='0.2.1',
    license='LICENSE',
    packages=[
      "sevimon",
      "sevimon.lib",
      "sevimon.lib.locale",
    ],
    package_dir={'sevimon': '', 'sevimon.lib': 'lib', 'sevimon.lib.locale': 'lib/locale'},
	package_data={'': ['*.txt', '*.md']},
    entry_points = {
        'console_scripts': [
            'sevicfg=sevimon.sevicfg:main',
            'sevimon=sevimon.sevimon:main',
            'sevistat=sevimon.sevistat:main',
        ],
    },
    url='https://github.com/ioctl-user/sevimon',
    description='Self Video Monitoring tool for facial muscles.',
    keywords='sevimon face emotion tension stress muscles wrinkles',
    install_requires=requirements,
)
